unit formPTChart;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VclTee.TeeGDIPlus, Data.DB,
  VCLTee.TeEngine, VCLTee.TeeURL, VCLTee.TeeSeriesTextEd, VCLTee.Series,
  Vcl.ExtCtrls, VCLTee.TeeProcs, VCLTee.Chart, VCLTee.DBChart, VCLTee.TeeTools, commasplit,
  Vcl.StdCtrls, Vcl.buttons, VCLTee.TeeMagnifyTool, VCLTee.TeeScrollPagerTool, System.StrUtils;

type
  TfrmPTChart = class(TForm)
    DBChart1: TDBChart;
    Series1: TLineSeries;
    Series2: TLineSeries;
    SeriesTextSource1: TSeriesTextSource;
    SeriesTextSource2: TSeriesTextSource;
    Series3: TLineSeries;
    SeriesTextSource3: TSeriesTextSource;
    SeriesTextSource4: TSeriesTextSource;
    CheckBox1: TCheckBox;
    RadioGroup1: TRadioGroup;
    procedure FormShow(Sender: TObject);
    procedure DBChart1BeforeDrawAxes(Sender: TObject);
    procedure DBChart1GetAxisLabel(Sender: TChartAxis;
      Series: TChartSeries; ValueIndex: Integer; var LabelText: string);
    procedure CheckBox1Click(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);

  private
    procedure ReadToArrays;
    function VDEL(aMD: double; VE: array of double): double;
    procedure SyncTopAxis;
    procedure SyncVertAxes;
    procedure SetRightAxisTitle;

    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmPTChart: TfrmPTChart;
  MD: array of double;
  VD: array of double;
  EL: array of double;

implementation

{$R *.dfm}

// ========== TfrmPTChart.FormShow =============================================
procedure TfrmPTChart.FormShow(Sender: TObject);
var
  i: integer;
  LStrList: TStringList;
  Lcs: TCommaSplitter;

begin
// Load Pressure vs Depth (Left Axis)
  Series1.DataSource := SeriesTextSource1;
  Series1.VertAxis := aBothVertAxis;

  with SeriesTextSource1 do
  begin
    FileName := 'C:\Code_Test_Progs\DeviatedPT\DeviatedPT.txt';
    FieldSeparator:=',';
    Fields.Clear;
    AddField('X',3);
    AddField('Y',1);
    Load;
  end;

// Load WHP vs Depth (Left Axis)
  Series2.DataSource := SeriesTextSource2;
  Series2.VertAxis := aBothVertAxis;

  with SeriesTextSource2 do
  begin
    FileName := 'C:\Code_Test_Progs\DeviatedPT\DeviatedPT.txt';
    FieldSeparator:=',';
    Fields.Clear;
    AddField('X',4);
    AddField('Y',1);
    Load;
  end;

// Load Temperature vs Depth (Left Axis)
  Series3.DataSource := SeriesTextSource3;
  Series3.VertAxis := aBothVertAxis;

  with SeriesTextSource3 do
  begin
    FileName := 'C:\Code_Test_Progs\DeviatedPT\DeviatedPT.txt';
    FieldSeparator:=',';
    Fields.Clear;
    AddField('X',2);
    AddField('Y',1);
    Load;
  end;

  ReadToArrays;

  SyncVertAxes;

  SetRightAxisTitle;

end;

// ========== TfrmPTChart.RadioGroup1Click =====================================
procedure TfrmPTChart.RadioGroup1Click(Sender: TObject);
begin
  SetRightAxisTitle;
  DBChart1.Repaint;
end;

// ========== TfrmPTChart.ReadToArrays =========================================
// Read Measured Depth/Vertical Depth/Elevation for well
procedure TfrmPTChart.ReadToArrays;
var
  Lsl: TStringList;
  i: integer;
  s: string;

begin
  Lsl := TStringList.Create;
  try
    Lsl.LoadFromFile('C:\Code_Test_Progs\DeviatedPT\AT-203_Deviation.csv');

    SetLength(MD,Lsl.Count);
    SetLength(VD,Lsl.Count);
    SetLength(EL,Lsl.Count);

    for i := 1 to Lsl.Count - 1 do
    begin
      s := Lsl[i];
      begin
        MD[i-1] := StrToFloat(Copy(s,1,Pos(',',s)-1));
        s := Copy(s,Pos(',',s) + 1,Length(s));
        VD[i-1] := StrToFloat(Copy(s,1,Pos(',',s)-1));
        EL[i-1] := StrToFloat(Copy(s,Pos(',',s) + 1,Length(s)));
      end;
    end;
  finally
    Lsl.Free;
  end;
end;

// ========== TfrmPTChart.VDEL =================================================
// Calculate Vertical Depth or Elevation for any specified depth
// Use VD or EL array as appropriate
function TfrmPTChart.VDEL(aMD: double; VE: array of double): double;
var
  i,j: Integer;

begin
  if aMD < MD[0] then
    j := 0
  else
  if aMD > MD[Length(MD)-2] then
    j := Length(MD) - 3
  else
  for i := 1 to Length(MD) - 2 do
  begin
    if aMD < MD[i] then
    begin
      j := i - 1;
      break;
    end;
  end;

  result := (aMD - MD[j])/(MD[j+1] - MD[j])*(VE[j+1]-VE[j]) + VE[j];

end;

// ========== TfrmPTChart.CheckBox1Click =======================================
// Doesn't work very well
procedure TfrmPTChart.CheckBox1Click(Sender: TObject);
begin
  if CheckBox1.Checked then
  begin
  end
  else
  begin
    DBChart1.Axes.Top.AutomaticMaximum := True;
    DBChart1.Axes.Top.AutomaticMinimum := True;
    DBChart1.Axes.Top.Automatic := True;
    DBChart1.Axes.Top.Increment := 0;
  end;
  DBChart1.Repaint;
end;

// ========== TfrmPTChart.DBChart1BeforeDrawAxes ===============================
// Now works on zoom, scroll etc
procedure TfrmPTChart.DBChart1BeforeDrawAxes(Sender: TObject);
begin
  if CheckBox1.Checked then
    SyncTopAxis;
end;

// ========== TfrmPTChart.DBChart1GetAxisLabel =================================
procedure TfrmPTChart.DBChart1GetAxisLabel(Sender: TChartAxis;
  Series: TChartSeries; ValueIndex: Integer; var LabelText: string);
var
  tmp: Double;

begin
  if Sender = DBChart1.Axes.Right then
  begin
    if (RadioGroup1.ItemIndex in [1,2]) then
    begin
      tmp:=StrToFloatDef(ReplaceStr(LabelText, FormatSettings.ThousandSeparator,''), 0);
      if (RadioGroup1.ItemIndex = 1) then // vertical depth
        tmp:=Round(VDEL(tmp, VD))
      else                                // elevation
        tmp:=Round(VDEL(tmp, EL));
      LabelText:=FormatFloat(Sender.AxisValuesFormat, tmp);
    end;
  end;
end;

// ========== TfrmPTChart.SyncVertAxes =========================================
procedure TfrmPTChart.SyncVertAxes;
begin
  DBChart1.RightAxis.AutomaticMinimum := DBChart1.LeftAxis.AutomaticMinimum;
  DBChart1.RightAxis.AutomaticMaximum := DBChart1.LeftAxis.AutomaticMaximum;
  DBChart1.RightAxis.Automatic := DBChart1.LeftAxis.Automatic;
  DBChart1.RightAxis.Minimum := DBChart1.LeftAxis.Minimum;
  DBChart1.RightAxis.Maximum := DBChart1.LeftAxis.Maximum;
  DBChart1.RightAxis.Increment := DBChart1.LeftAxis.Increment;
  DBChart1.RightAxis.Inverted := DBChart1.LeftAxis.Inverted;
  DBChart1.RightAxis.Logarithmic := DBChart1.LeftAxis.Logarithmic;
end;

// ========== TfrmPTChart.SyncTopAxis ==========================================
// Uses bottom axis increments to locate top axis labels
procedure TfrmPTChart.SyncTopAxis;
var
  i: integer;
  Tval, TPos, TMin, TMax, TDiff, TInc, BMin, BMax, BDiff, BInc: double;

begin
  with DBChart1.Axes do
  begin
    Top.Items.Clear;
    TMin := Top.Minimum;
    TMax := Top.Maximum;
    TDiff := TMax - Tmin;
    TInc := Top.Increment;
    BMin := Bottom.Minimum;
    BMax := Bottom.Maximum;
    BDiff := BMax - BMin;
    BInc := Bottom.CalcIncrement;

    i:=Round(Round(BMin/BInc) * BInc);
    repeat
      Tval := Round((i - BMin)/BDiff*TDiff + TMin);
      TPos := (i - Bmin)/BDiff*TDiff+TMin;
      Top.Items.Add(TPos, FloatToStr(Tval));  // adding custom label
      Inc(i, Round(Bottom.CalcIncrement));
    until i>=Round(BMax);
  end;
end;

// ========== TfrmPTChart.SetRightAxisTitle ====================================
procedure TfrmPTChart.SetRightAxisTitle;
begin
  if (RadioGroup1.ItemIndex = 0) then
    DBChart1.RightAxis.Title.Text := 'Measured Depth (m)'
  else if (RadioGroup1.ItemIndex = 1) then
    DBChart1.RightAxis.Title.Text := 'Vertical Depth (m)'
  else if (RadioGroup1.ItemIndex = 2) then
    DBChart1.RightAxis.Title.Text := 'Elevation (m)';
end;

end.
